/*
 * Created on 29.09.2004
 */

package org.test.comp.remote;

import javax.swing.JOptionPane;

import org.test.comp.OOGlobalEventConfigurator;

import com.sun.star.beans.XPropertySet;
import com.sun.star.bridge.XBridge;
import com.sun.star.bridge.XBridgeFactory;
import com.sun.star.connection.XConnection;
import com.sun.star.connection.XConnector;
import com.sun.star.lang.EventObject;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XEventListener;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 * @author Oliver Brinzing
 */

public class RemoteConnection {

    private XComponentContext xRemoteContext = null;

    public RemoteConnection(String sUnoUrl) {

        try {
            xRemoteContext = getRemoteContext(sUnoUrl);

            if (xRemoteContext != null)
                new OOGlobalEventConfigurator(xRemoteContext).trigger("");

        } catch (com.sun.star.connection.NoConnectException e) {
            JOptionPane.showMessageDialog(null, "parameter:\n"
                    + "soffice.exe -accept=" + sUnoUrl.substring(4),
                    "cannot find openoffice ...",
                    JOptionPane.INFORMATION_MESSAGE);
            System.exit(1);
        } catch (com.sun.star.lang.DisposedException e) {
            connectionDisposed();
        } catch (java.lang.Exception e) {
            e.printStackTrace();
        }
    }

    private void connectionDisposed() {
        JOptionPane.showMessageDialog(null, "connection disposed ...",
                " disposed", JOptionPane.INFORMATION_MESSAGE);
        xRemoteContext = null;
        System.exit(0);
    }

    protected XComponentContext getRemoteContext(String sUnoUrl)
            throws java.lang.Exception {

        if (xRemoteContext != null)
            return xRemoteContext;

        XComponentContext xLocalContext = null;

        /*
         * Bootstraps a component context with the jurt base components
         * registered. Component context to be granted to a component for
         * running. Arbitrary values can be retrieved from the context.
         */

        xLocalContext = com.sun.star.comp.helper.Bootstrap
                .createInitialComponentContext(null);
        /*
         * Gets the service manager instance to be used (or null). This method
         * has been added for convenience, because the service manager is a
         * often used object.
         */
        XMultiComponentFactory xLocalServiceManager = xLocalContext
                .getServiceManager();

        // create connector service
        Object objConnector = xLocalServiceManager.createInstanceWithContext(
                "com.sun.star.connection.Connector", xLocalContext);

        XConnector xConnector = (XConnector) UnoRuntime.queryInterface(
                XConnector.class, objConnector);

        String token[] = parseUnoURL(sUnoUrl);

        // connect (socket,host=...,port=...)
        XConnection xConnection = xConnector.connect(token[0]);

        // create BridgeFactory ...
        Object objBridge = xLocalServiceManager.createInstanceWithContext(
                "com.sun.star.bridge.BridgeFactory", xLocalContext);

        XBridgeFactory xBridgeFactory = (XBridgeFactory) UnoRuntime
                .queryInterface(XBridgeFactory.class, objBridge);

        // create Bridge (nameless, urp, socket,..., no instance provider)
        XBridge xBridge = xBridgeFactory.createBridge("", token[1],
                xConnection, null);

        // query XComponent Interface
        XComponent xComponent = (XComponent) UnoRuntime.queryInterface(
                XComponent.class, xBridge);

        // add EventListener
        xComponent.addEventListener(new RemoteConnectionListener());

        // get the remote ServiceManager
        Object objInitial = xBridge.getInstance(token[2]);

        // Query for the XPropertySet interface.
        XPropertySet xPropertySet = (XPropertySet) UnoRuntime.queryInterface(
                XPropertySet.class, objInitial);

        // Get the default context from the office server.
        Object oDefaultContext = xPropertySet
                .getPropertyValue("DefaultContext");

        // Query for the interface XComponentContext.
        xRemoteContext = (XComponentContext) UnoRuntime.queryInterface(
                XComponentContext.class, oDefaultContext);

        return xRemoteContext;
    }

    private String[] parseUnoURL(String sUnoURL) {
        if (sUnoURL == null || sUnoURL == "" || !sUnoURL.startsWith("uno:"))
            return null;

        return sUnoURL.substring(4).split(";");
    }

    private class RemoteConnectionListener implements XEventListener {
        public void disposing(EventObject arg0) {
            connectionDisposed();
        }
    }
}